/* $Id: timer.h,v 1.3 2006/01/18 12:04:05 cvsroot Exp $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_TIMER_INCLUDED
#define GENLIB2_TIMER_INCLUDED

#include <climits>
#include <ctime>


namespace GenLib2
{

/** Timer class.
 *  Simple stopwatch using ISO C function clock.
 *  Be carefull, the measured CPU-time might be implementation dependent.
 *  @author Dirk Stallmann
 */
class Timer
{
public:
	/// constructor
	Timer() : _running(false), _accu(0), _last_time(0) {}

	/// reset timer (set elapsed time to 0)
	void reset() { _running = false; _last_time = _accu = 0; }

	/// start timer
	void start()
	{
		if (!_running)
			{
			_last_time = clock();
			_running = true;
			}
	}

	/// stop timer and return elapsed time in seconds
	double stop()
	{
		if (_running)
			{
			_accu += clock() - _last_time;
			_running = false;
			}
		return double(_accu) / double(CLOCKS_PER_SEC);
	}

	/// return elapsed time in seconds
	double time()
	{
		if (_running)
			{
			_accu += clock() - _last_time;
			_last_time = clock();
			}
		return double(_accu) / double(CLOCKS_PER_SEC);
	}
    
private:
	/// timer running
	bool _running;

	/// accumulator
	clock_t _accu;

	/// last time
	clock_t _last_time;
};


// Implementations


} // namespace GenLib2

#endif /* GENLIB2_TIMER_INCLUDED */
